@@ -11,7 +11,7 @@ from tqdm import trange
 class Tomography(Experiment):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.patternList = None
+        self.patterns = None
 
     # PUBLIC METHODS
     def check(self):
@@ -124,32 +124,37 @@ class Tomography(Experiment):
         plt.show()
 
     def plot_angle_frequency_distribution(self):
-        """
-        Plots the distribution of angles and spatial frequencies from a list of patterns.
-        Args:
-            patterns (list): List of strings in the format "hex_angle".
-            num_elements (int): Number of elements in each pattern (default is 192).
-        """
-        if self.patternList is None:
-            raise ValueError("patternList is not initialized. Please load or generate the active list first.")
-        
+        if self.patterns is None:
+            raise ValueError("patterns is not initialized. Please load or generate the active list first.")
+
         num_elements = self.params.acoustic['num_elements']
-        divs = [d for d in range(2, num_elements + 1) if num_elements % d == 0 and d % 2 == 0]
+        divs = sorted([d for d in range(2, num_elements + 1) if num_elements % d == 0 and d % 2 == 0])
+        if num_elements not in divs:
+            divs.append(num_elements)
+        divs.sort()
+
         angles = []
         freqs = []
+
         for p in self.patterns:
-            hex_part, angle_str = p.split('_')
+            # Extraire la chaîne "hexa_XXX" depuis le dictionnaire
+            file_name = p["fileName"]
+            hex_part, angle_str = file_name.split('_')  # Split sur le dictionnaire corrigé
+
             # Récupérer l'angle
             sign = -1 if angle_str[0] == '1' else 1
             angle = sign * int(angle_str[1:])
             angles.append(angle)
-            # Récupérer fréquence spatiale
-            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(num_elements)])
 
-            # Chercher la plus petite taille de bloc divisant num_elements
+            # Récupérer la fréquence spatiale
+            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(num_elements)])
+            if np.all(bits == 1):  # Cas "tous activés"
+                freqs.append(num_elements)
+                continue
+
             for block_size in divs:
                 half_block = block_size // 2
-                block = np.array([0]*half_block + [1]*half_block)
+                block = np.array([0] * half_block + [1] * half_block)
                 reps = num_elements // block_size
                 pattern_check = np.tile(block, reps)
                 if any(np.array_equal(np.roll(pattern_check, shift), bits) for shift in range(block_size)):
@@ -157,32 +162,42 @@ class Tomography(Experiment):
                     break
             else:
                 freqs.append(None)
-        # Filtrer les valeurs None
+
         freqs = [f for f in freqs if f is not None]
+
         # Plot
         fig, axes = plt.subplots(1, 2, figsize=(12, 5))
+
         # Histogramme des angles
         axes[0].hist(angles, bins=np.arange(-20.5, 21.5, 1), color='skyblue', edgecolor='black', rwidth=0.8)
         axes[0].set_xlabel("Angle (°)")
         axes[0].set_ylabel("Nombre de patterns")
         axes[0].set_title("Distribution des angles")
         axes[0].set_xticks(np.arange(-20, 21, 2))
+
         # Histogramme des fréquences spatiales
-        bins = np.append(divs, 193)
-        axes[1].hist(freqs, bins=bins, color='salmon', edgecolor='black', rwidth=0.8)
-        axes[1].set_xscale('log')
-        axes[1].set_xticks(divs)
+        unique_freqs, freq_counts = np.unique(freqs, return_counts=True)
+        x_pos = np.arange(len(divs))
+        for freq, count in zip(unique_freqs, freq_counts):
+            idx = divs.index(freq)
+            axes[1].bar(x_pos[idx], count, color='salmon', edgecolor='black', width=0.8)
+
+        axes[1].set_xticks(x_pos)
         axes[1].set_xticklabels(divs)
-        axes[1].set_xlabel("Taille du bloc / fréquence spatiale")
+        axes[1].set_xlabel("Taille du bloc (fréquence spatiale)")
         axes[1].set_ylabel("Nombre de patterns")
         axes[1].set_title("Distribution des fréquences spatiales")
+
         plt.tight_layout()
         plt.show()
 
+
+
+
     def loadActiveList(self, fieldParamPath):
         if not os.path.exists(fieldParamPath):
             raise FileNotFoundError(f"Field parameter file {fieldParamPath} not found.")
-        patternList = []
+        patterns = []
         with open(fieldParamPath, 'r') as file:
             lines = file.readlines()
             for line in lines:
@@ -190,14 +205,14 @@ class Tomography(Experiment):
                 if not line:
                     continue
                 if "_" in line and all(c in "0123456789abcdefABCDEF" for c in line.split("_")[0]):
-                    patternList.append({"fileName": line})
+                    patterns.append({"fileName": line})
                     continue
                 try:
                     parsed = eval(line, {"__builtins__": None})
                     if isinstance(parsed, tuple) and len(parsed) == 2:
                         coords, angles = parsed
                         for angle in angles:
-                            patternList.append({
+                            patterns.append({
                                 "space_0": coords[0],
                                 "space_1": coords[1],
                                 "move_head_0_2tail": coords[2],
@@ -208,7 +223,31 @@ class Tomography(Experiment):
                         raise ValueError("Ligne inattendue (pas un tuple de deux éléments)")
                 except Exception as e:
                     print(f"Erreur de parsing sur la ligne : {line}\n{e}")
-        self.patternList = patternList
+        self.patterns = patterns
+
+    def saveActiveList(self, filePath):
+        """
+        Sauvegarde la liste des patterns dans un fichier texte.
+        Args:
+            filePath (str): Chemin du fichier de sortie.
+        """
+        with open(filePath, 'w') as file:
+            for pattern in self.patterns:
+                if "fileName" in pattern:
+                    # Cas 1 : Pattern simple (format "hexa_XXX")
+                    file.write(f"{pattern['fileName']}\n")
+                else:
+                    # Cas 2 : Pattern avec paramètres (format tuple)
+                    coords = (
+                        pattern["space_0"],
+                        pattern["space_1"],
+                        pattern["move_head_0_2tail"],
+                        pattern["move_tail_1_2head"]
+                    )
+                    angles = [pattern["angle"]]  # Supposons que chaque pattern a un seul angle
+                    line = f"({coords}, {angles})\n"
+                    file.write(line)
+
 
     def generateActiveList(self, N):
         """
@@ -243,10 +282,11 @@ class Tomography(Experiment):
             print(f"Aucun diviseur pair trouvé pour num_elements = {num_elements}")
             return []
 
-        results = []
+        # 2. Utiliser un ensemble pour suivre les patterns uniques
+        unique_patterns = set()
 
-        # 2. Générer N patterns en tirant aléatoirement parmi les divs et les angles
-        for _ in range(N):
+        # 3. Générer jusqu'à N patterns uniques
+        while len(unique_patterns) < N:
             # Tirer un diviseur aléatoire (y compris num_elements)
             block_size = np.random.choice(divs)
 
@@ -268,50 +308,75 @@ class Tomography(Experiment):
             angle = np.random.choice(angle_choices)
             pair = f"{hex_pattern}_{format_angle(angle)}"
 
-            results.append(pair)
-
-        return results
-
+            # Ajouter à l'ensemble (les doublons sont automatiquement ignorés)
+            unique_patterns.add(pair)
 
+        # 4. Convertir en liste de dictionnaires avec la clé "fileName"
+        patterns = [{"fileName": pair} for pair in unique_patterns]
 
+        # 5. Retourner exactement N patterns (on a déjà vérifié la taille avec while)
+        return patterns[:N]  # Par sécurité, même si len(unique_patterns) == N
 
     def _check_patterns(self, patterns):
-        for p in patterns:
-            hex_part = p.split('_')[0]
-            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(self.params.acoustic['num_elements'])])
-            if len(bits) != self.params.acoustic['num_elements']:
-                print(f"Erreur longueur: {p}")
+        # 1. Vérifier les doublons (basé sur "fileName")
+        file_names = [p["fileName"] for p in patterns]
+        if len(file_names) != len(set(file_names)):
+            # Trouver les doublons
+            from collections import Counter
+            file_counts = Counter(file_names)
+            duplicates = [fn for fn, count in file_counts.items() if count > 1]
+            for dup in duplicates:
+                print(f"Erreur : Doublon détecté pour {dup}")
+            return False
+
+        # 2. Vérifier chaque pattern individuellement
+        num_elements = self.params.acoustic['num_elements']
+        for pattern in patterns:
+            hex_part, angle_str = pattern["fileName"].split('_')
+            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(num_elements)])
+
+            # Vérifier la longueur
+            if len(bits) != num_elements:
+                print(f"Erreur longueur: {pattern['fileName']}")
                 return False
+
+            # Cas spécial : pattern "tous activés"
             if np.all(bits == 1):
                 continue
-            if np.sum(bits) != self.params.acoustic['num_elements'] // 2:
-                print(f"Erreur équilibre 0/1: {p}")
+
+            # Vérifier l'équilibre 0/1
+            if np.sum(bits) != num_elements // 2:
+                print(f"Erreur équilibre 0/1: {pattern['fileName']}")
                 return False
+
+            # Vérifier la régularité
             valid = False
-            for block_size in range(2, self.params.acoustic['num_elements']+1, 2):
-                if self.params.acoustic['num_elements'] % block_size != 0:
-                    continue
+            divs = [d for d in range(2, num_elements + 1) if num_elements % d == 0 and d % 2 == 0]
+            for block_size in divs:
                 half_block = block_size // 2
-                block = np.array([0]*half_block + [1]*half_block)
-                reps = self.params.acoustic['num_elements'] // block_size
+                block = np.array([0] * half_block + [1] * half_block)
+                reps = num_elements // block_size
                 expected_pattern = np.tile(block, reps)
                 if any(np.array_equal(np.roll(expected_pattern, shift), bits) for shift in range(block_size)):
                     valid = True
                     break
             if not valid:
-                print(f"Erreur régularité: {p}")
+                print(f"Erreur régularité: {pattern['fileName']}")
                 return False
+
         return True
 
+
+
     # PRIVATE METHODS
     def _generateAcousticFields_STRUCT_CPU(self, fieldDataPath=None, show_log=False):
-        if self.patternList is None:
-            raise ValueError("patternList is not initialized. Please load or generate the active list first.")
+        if self.patterns is None:
+            raise ValueError("patterns is not initialized. Please load or generate the active list first.")
         listAcousticFields = []
-        progress_bar = trange(0, len(self.patternList), desc="Generating acoustic fields")
+        progress_bar = trange(0, len(self.patterns), desc="Generating acoustic fields")
         for i in progress_bar:
             memory = psutil.virtual_memory()
-            pattern = self.patternList[i]
+            pattern = self.patterns[i]
             if "fileName" in pattern:
                 AcousticField = StructuredWave(fileName=pattern["fileName"], params=self.params)
             else:
